#version 330
#define MAX_POWERUPS 10

uniform mat4 uni_ModelViewMatrix;
uniform mat4 uni_ProjMatrix;
uniform mat3 uni_NormalMatrix; //transforms normals to VIEW space
uniform mat4 uni_ModelLightMatrix; //=light view matrix * model matrix
uniform mat4 uni_LightNormMatrix; //combines light projection and normalization matrix
uniform vec4 uni_TextureMatrix1[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix2[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix3[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_TextureMatrix4[MAX_POWERUPS]; //projector projection + normalization matrix
uniform vec4 uni_ModelProjectorMatrix1[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix2[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix3[MAX_POWERUPS]; //projector view matrix * model matrix
uniform vec4 uni_ModelProjectorMatrix4[MAX_POWERUPS]; //projector view matrix * model matrix

in vec3 in_Vertex;
in vec3 in_Normal;
in vec2 in_Tex;

out vec2 var_TexCoord;
out vec3 var_Normal; //transformed normal in view space
out vec3 var_EyeVec; //from point to eye (in view space, = -pos in view space), not normalized
out vec4 var_SMTexCoords; //shadow map texture coordinates
out vec4 var_TexCoordProj[MAX_POWERUPS]; //projective texturing coordinates

void main(void)
{
	//transform normals
	var_Normal = normalize(uni_NormalMatrix * in_Normal);

	//dont need to calc light direction for parallel light source

	// interpolate tex coords
	var_TexCoord = in_Tex;

	vec4 pos4 = vec4(in_Vertex,1);
	vec4 posViewCoords = uni_ModelViewMatrix * pos4;
	var_EyeVec = -vec3(posViewCoords);

	//final vtx position
	gl_Position =  uni_ProjMatrix * posViewCoords;

	//texture coords for projective texturing
	mat4 uni_TextureMatrix[MAX_POWERUPS];
	mat4 uni_ModelProjectorMatrix[MAX_POWERUPS];

    for(int i=0;i<MAX_POWERUPS;i++) 
	{		
		uni_TextureMatrix[i] = mat4(uni_TextureMatrix1[i],uni_TextureMatrix2[i],uni_TextureMatrix3[i],uni_TextureMatrix4[i]);
		uni_ModelProjectorMatrix[i] = mat4(uni_ModelProjectorMatrix1[i],uni_ModelProjectorMatrix2[i],uni_ModelProjectorMatrix3[i],uni_ModelProjectorMatrix4[i]);
		var_TexCoordProj[i] = uni_TextureMatrix[i] * uni_ModelProjectorMatrix[i] * pos4;
	}

	//transform vtx to light coords
	var_SMTexCoords = uni_LightNormMatrix * uni_ModelLightMatrix * pos4;
}